@@ -64,6 +64,8 @@ namespace cmdShim {
      * Path to the Node.js executable
      */
     nodeExecPath?: string
+
+    prependToPath?: string
   }
 }
 type Options = cmdShim.Options
@@ -148,7 +150,6 @@ function ingestOptions (opts?: Options): InternalOptions {
  */
 async function cmdShim (src: string, to: string, opts?: Options): Promise<void> {
   const opts_ = ingestOptions(opts)
-  await opts_.fs_.stat(src)
   await cmdShim_(src, to, opts_)
 }
 
@@ -262,26 +263,49 @@ interface RuntimeInfo {
  * @return Promise of infomation of runtime of `target`.
  */
 async function searchScriptRuntime (target: string, opts: InternalOptions): Promise<RuntimeInfo> {
-  const data = await opts.fs_.readFile(target, 'utf8')
+  try {
+    const data = await opts.fs_.readFile(target, 'utf8')
 
-  // First, check if the bin is a #! of some sort.
-  const firstLine = data.trim().split(/\r*\n/)[0]
-  const shebang = firstLine.match(shebangExpr)
-  if (!shebang) {
-    // If not, infer script type from its extension.
-    // If the inference fails, it's something that'll be compiled, or some other
-    // sort of script, and just call it directly.
-    const targetExtension = path.extname(target).toLowerCase()
-    return {
-      // undefined if extension is unknown but it's converted to null.
-      program: extensionToProgramMap.get(targetExtension) || null,
-      additionalArgs: ''
+    // First, check if the bin is a #! of some sort.
+    const firstLine = data.trim().split(/\r*\n/)[0]
+    const shebang = firstLine.match(shebangExpr)
+    if (!shebang) {
+      // If not, infer script type from its extension.
+      // If the inference fails, it's something that'll be compiled, or some other
+      // sort of script, and just call it directly.
+      const targetExtension = path.extname(target).toLowerCase()
+      return {
+        // undefined if extension is unknown but it's converted to null.
+        program: extensionToProgramMap.get(targetExtension) || null,
+        additionalArgs: ''
+      }
     }
+    return {
+      program: shebang[1],
+      additionalArgs: shebang[2]
+    }
+  } catch (err: any) {
+    if (!isWindows() || err.code !== 'ENOENT') throw err
+    if (await opts.fs_.stat(`${target}${getExeExtension()}`)) {
+      return {
+        program: null,
+        additionalArgs: '',
+      }
+    }
+    throw err
   }
-  return {
-    program: shebang[1],
-    additionalArgs: shebang[2]
+}
+
+function getExeExtension (): string {
+  let cmdExtension
+
+  if (process.env.PATHEXT) {
+    cmdExtension = process.env.PATHEXT
+      .split(path.delimiter)
+      .find(ext => ext.toLowerCase() === '.exe')
   }
+
+  return cmdExtension || '.exe'
 }
 
 /**
@@ -327,6 +351,7 @@ function generateCmdShim (src: string, to: string, opts: InternalOptions): strin
   let prog = opts.prog
   let args = opts.args || ''
   const nodePath = normalizePathEnvVar(opts.nodePath).win32
+  const prependToPath = normalizePathEnvVar(opts.prependToPath).win32
   if (!prog) {
     prog = quotedPathToTarget
     args = ''
@@ -349,6 +374,9 @@ function generateCmdShim (src: string, to: string, opts: InternalOptions): strin
   //   node "%~dp0\.\node_modules\npm\bin\npm-cli.js" %*
   // )
   let cmd = '@SETLOCAL\r\n'
+  if (prependToPath) {
+    cmd += `@SET "PATH=${prependToPath}:%PATH%"\r\n`
+  }
   if (nodePath) {
       cmd += `\
 @IF NOT DEFINED NODE_PATH (\r
@@ -429,7 +457,12 @@ case \`uname\` in
 esac
 
 `
-  if (opts.nodePath) {
+  if (opts.prependToPath) {
+    sh += `\
+export PATH="${opts.prependToPath}:$PATH"
+`
+  }
+  if (shNodePath) {
       sh += `\
 if [ -z "$NODE_PATH" ]; then
   export NODE_PATH="${shNodePath}"
@@ -474,9 +507,12 @@ function generatePwshShim (src: string, to: string, opts: InternalOptions): stri
   shTarget = shTarget.split('\\').join('/')
   const quotedPathToTarget = path.isAbsolute(shTarget) ? `"${shTarget}"` : `"$basedir/${shTarget}"`
   let args = opts.args || ''
-  let normalizedPathEnvVar = normalizePathEnvVar(opts.nodePath)
-  const nodePath = normalizedPathEnvVar.win32
-  const shNodePath = normalizedPathEnvVar.posix
+  let normalizedNodePathEnvVar = normalizePathEnvVar(opts.nodePath)
+  const nodePath = normalizedNodePathEnvVar.win32
+  const shNodePath = normalizedNodePathEnvVar.posix
+  let normalizedPrependPathEnvVar = normalizePathEnvVar(opts.prependToPath)
+  const prependPath = normalizedPrependPathEnvVar.win32
+  const shPrependPath = normalizedPrependPathEnvVar.posix
   if (!pwshProg) {
     pwshProg = quotedPathToTarget
     args = ''
@@ -524,27 +560,41 @@ function generatePwshShim (src: string, to: string, opts: InternalOptions): stri
 $basedir=Split-Path $MyInvocation.MyCommand.Definition -Parent
 
 $exe=""
-${opts.nodePath ? `\
-$pathsep=":"
+${(nodePath || prependPath) ? '$pathsep=":"\n' : ''}\
+${nodePath ? `\
 $env_node_path=$env:NODE_PATH
 $new_node_path="${nodePath}"
 ` : ''}\
+${prependPath ? `\
+$env_path=$env:PATH
+$prepend_path="${prependPath}"
+` : ''}\
 if ($PSVersionTable.PSVersion -lt "6.0" -or $IsWindows) {
   # Fix case when both the Windows and Linux builds of Node
   # are installed in the same directory
   $exe=".exe"
-${opts.nodePath ? '  $pathsep=";"\n' : ''}\
+${(nodePath || prependPath) ? '  $pathsep=";"\n' : ''}\
 }`
-  if (opts.nodePath) {
+  if (shNodePath || shPrependPath) {
     pwsh += `\
  else {
-  $new_node_path="${shNodePath}"
+${shNodePath ? `  $new_node_path="${shNodePath}"\n` : ''}\
+${shPrependPath ? `  $prepend_path="${shPrependPath}"\n` : ''}\
 }
+`
+  }
+  if (shNodePath) {
+    pwsh += `\
 if ([string]::IsNullOrEmpty($env_node_path)) {
   $env:NODE_PATH=$new_node_path
 } else {
   $env:NODE_PATH="$env_node_path$pathsep$new_node_path"
 }
+`
+  }
+  if (opts.prependToPath) {
+    pwsh += `
+$env:PATH="$prepend_path$pathsep$env:PATH"
 `
   }
   if (pwshLongProg) {
@@ -567,7 +617,8 @@ if (Test-Path ${pwshLongProg}) {
   }
   $ret=$LASTEXITCODE
 }
-${opts.nodePath ? '$env:NODE_PATH=$env_node_path\n' : ''}\
+${nodePath ? '$env:NODE_PATH=$env_node_path\n' : ''}\
+${prependPath ? '$env:PATH=$env_path\n' : ''}\
 exit $ret
 `
   } else {
@@ -578,7 +629,8 @@ if ($MyInvocation.ExpectingInput) {
 } else {
   & ${pwshProg} ${args} ${shTarget} ${progArgs}$args
 }
-${opts.nodePath ? '$env:NODE_PATH=$env_node_path\n' : ''}\
+${nodePath ? '$env:NODE_PATH=$env_node_path\n' : ''}\
+${prependPath ? '$env:PATH=$env_path\n' : ''}\
 exit $LASTEXITCODE
 `
   }
@@ -601,7 +653,7 @@ interface NormalizedPathEnvVar {
   [index:number]: {win32:string,posix:string}
 }
 function normalizePathEnvVar (nodePath: undefined | string | string[]): NormalizedPathEnvVar {
-  if (!nodePath) {
+  if (!nodePath || !nodePath.length) {
     return {
       win32: '',
       posix: ''
